% Duck World - a demonstrative Amzi! Prolog application
%
% Copyright (c) 1994-1995, Amzi! inc.
% All rights reserved
%
% ducky.pro - the Duck World program all in a single file, designed for
% simple interpretive use.
%
% Tiba muudetud - tehtud normaalse tekstika sarnaseks


main :-
	init_state,
	write($ Welcome to Duck World $),nl,
	write($ The goal of the game - make yourself an omelette $),nl,
	nl,
	look,nl,
	write($ Use 'help' to see possible commands$),nl,
	go.

% initialize the dynamic database.  This is unnecessary for interpreted use
% but is needed to make the program work compiled.  For interpreted use
% the simple statements
%
% loc(egg,pen).
% loc(ducks,pen).
% loc(you,house).
%
% are all that are required.  This is because all of the clauses of a
% program are in the dynamic database when interpreted.

init_state :-
	retractall(loc(_,_)),
	retractall(cantake(_)),
	retractall(usercommand(_)),
	retractall(canchase(_)),
	retractall(chaseinto(_,_)),
	retractall(canseetheducks(_)),
	retractall(use(_,_)),
	retractall(marksmanmode),
	retractall(losses(_)),
	assert(loc(egg,pen)),
	assert(loc(ducks,pen)),
	assert(loc(you,house)),
	assert(loc(rifle,house)),
	assert(loc(fox,woods)),
	assert(loc(beercan,backyard)),
	assert(cantake(egg)),
	assert(cantake(rifle)),
	assert(cantake(beercan)),
	assert(usercommand(go(_))),
	assert(usercommand(look)),
	assert(usercommand(take(_))),
	assert(usercommand(drop(_))),
	assert(usercommand(chase(_))),
	assert(usercommand(use(_))),
	assert(usercommand(help)),
	assert(canchase(ducks)),
	assert(canchase(fox)),
	assert(chaseinto(ducks, pen)),
	assert(chaseinto(fox, woods)),
	assert(canseetheducks(yard)),
	assert(canseetheducks(pen)),
	assert(canuse(rifle)),
	assert(canuse(egg)),
	assert(canuse(rifle, yard)),
	assert(canuse(rifle, backyard)),
	assert(canuse(rifle, woods)),
	assert(canuse(egg, house)),
	assert(losses(0)).

% control loop

go :- done.
go :-
	write($>> $),
	read(X),
	!,
	inputhandler(X).

inputhandler(quit) :-
	write($ Thank you for playing Duck World. $), nl.

inputhandler(X) :-
	usercommand(X),
	call(X),
	demons,
	go.

inputhandler(X) :-
	write($ Puhutteko suomea? $), nl,
	go.



% utilities functions and static predicates, seeing as these will not
% change they are OK to be compiled.

nextto(backyard, house).
nextto(pen, yard).
nextto(yard, house).
nextto(yard, woods).

connect(X,Y) :- nextto(X,Y).
connect(X,Y) :- nextto(Y,X).

move(Item, Place) :-
	retract(loc(Item, _)),
	assert(loc(Item,Place)).


% the commands that the user will type to manipulate the game.

help :-
	write($ The usable commands are : $), nl,
	write($      go(Location) - goes to the Location$), nl,
	write($      look         - gives you a description of the surroundings$), nl,
	write($      take(Item)   - picks up the Item$), nl,
	write($      drop(Item)   - drops the Item$), nl,
	write($      chase(Who)   - chases the Who back to where it belongs$), nl,
	write($      use(Item)    - uses the Item$), nl,
	write($      quit         - quits the game$), nl,
	write($      help         - displays this info$), nl, nl.


look :-
	loc(you, CurrentLoc),
	write($ You are in the $), write(CurrentLoc), write($.$), nl,
	findall(FThings, loc(FThings, CurrentLoc), Things),
	findall(FExits, connect(FExits, CurrentLoc), Exits),
	findall(FInventory, loc(FInventory, you), Inventory),
	write($ Exits are : $), writelist(Exits),nl,
	write($ You can see : $), writelist(Things),nl,
	write($ You are carrying : $), writelist(Inventory),nl.

writelist([]) :-
	write($nothing$),
	!.

writelist([First]) :-
	write(First),
	!.

writelist([First, Second]) :-
	write(First), write($, $),
	write(Second),
	!.
	

writelist([First|Others]) :-
	write(First), write($, $),
	writelist(Others).


go(woods) :-
	loc(you, L),
	connect(L, woods),
	not loc(rifle, you),
	write($ You dare not go into the woods without a rifle. $), nl.


go(X) :-
	loc(you, L),
	connect(L, X),
	move(you, X),
	look.

go(X) :-
	loc(you, X),
	write($ You already are in the $), write(X), write($.$), nl.

go(X) :-
	loc(you, L),
	write($ You can't get to the $), write(X), write($ from the $), 
	write(L), write($.$),nl.

chase(X) :-
	loc(X, L),
	loc(you, L),
	canchase(X),
	chaseinto(X, WhereTo),
	move(X, WhereTo),
	write($ You chase the $), write(X), write($ into the $), write(WhereTo), write($.$), nl.

chase(X) :-
	loc(X, L),
	loc(you, L),
	canchase(X),
	not chaseinto(X, _),
	write($ You have no place to chase the $), write(X), write($ to.$), nl.

chase(X) :-
	loc(X, L),
	loc(you, L),
	not canchase(X),
	write($ You cannot chase the $), write(X), write($.$), nl.

chase(X) :-
	write($ You cannot see the $), write(X), write($.$), nl.


take(X) :-
	cantake(X),
	loc(you, L),
	loc(X, L),
	move(X, you),
	write($ You now have the $), write(X), write($.$), nl.

take(X) :-
	not cantake(X),
	loc(you, L),
	loc(X, L),
	write($ You cannot take the $), write(X), write($.$), nl.

take(X) :-
	write($ There is no $), write(X), write($ here.$), nl.

drop(X) :-
	loc(X, you),
	loc(you, L),
	move(X, L),
	write($ You drop the $), write(X), write($.$), nl.

drop(X) :-
	write($ You are not carrying the $), write(X), write($.$), nl.

use(What) :-
	loc(What, you),
	loc(you, Here),
	canuse(What),
	canuse(What, Here),
	use2(What, Here).

use(What) :-
	loc(What, you),
	loc(you, Here),
	canuse(What),
	not canuse(What, Here),
	write($ You cannot use the $), write(What), write($ in the $), write(Here), write($.$), 
	nl.

use(What) :-
	loc(What, you),
	not canuse(What),
	write($ You cannot use the $), write(What), write($.$), nl.

use(What) :-
	canuse(What),
	not loc(What, you),
	write($ You are not carrying the $), write(What), write($.$), nl.

use2(rifle, Here) :-
	not loc(fox, Here),
	loc(beercan, Here),
	write($ You spend some time shooting holes in the beercan. There is not much left of the beercan,$), 
	nl, write($ but your aim has improved considerably.$),
	nl,
	move(beercan, trash),
	assert(loc(debris, Here)),
	assert(marksmanmode).

use2(rifle, Here) :-
	not loc(fox, Here),
	write($ Whom would you shoot? Yourself?$), nl.

use2(rifle, Here) :-
	loc(fox, Here),
	marksmanmode,
	write($ BANG! You shoot the fox.$), nl,
	assert(loc(corpse, Here)),
	move(fox, trash).

use2(rifle, _) :-
	not marksmanmode,
	write($ BANG! You use the rifle and almost blow your head off. You are not much of a shooter.$),
	nl, 
	((loc(fox, yard), loc(you, yard), write($ The ruckus has scared the fox back into the woods.$), 
	move(fox, woods),nl);true).

use2(egg, house) :-
	loc(ducks, yard),
	write($ You are unable to concentrate on the process of cooking.$), nl,
	write($ Your mind is uneasy, thinking of the poor ducklings roaming in the yard.$), nl.

use2(egg, house) :-
	not loc(fox, trash),
	write($ You are unable to concentrate on the process of cooking.$), nl,
	write($ Your mind is uneasy, thinking of the wicked fox roaming free.$), nl.

use2(egg, _) :-
	move(egg,trash),
	assert(loc(omelette, house)).

% The demon predicates, waiting for certain situations to occur.

demons :-
	ducks,
	fox.

ducks :-
	loc(ducks, pen),
	loc(you, pen),
	move(ducks, yard),
	write($ The ducks have run into the yard. $), nl,
	((loc(fox, woods),
	move(fox, yard),
	write($ The fox has crept into the yard. $), nl);true).

ducks.

fox :-
	loc(ducks, yard),
	loc(you, You),
	not canseetheducks(You),
	not loc(fox, trash),
	not loc(fox, You),
	write($ The fox has taken a duck. $), nl,
	losses(Losses), NewLosses is Losses + 1,
	retract(losses(Losses)), assert(losses(NewLosses)).
fox.


% end condition

done :-
	loc(you, house),
	loc(omelette, house),
	write($ You have made yourself an omelette. Enjoy! $), nl,
	losses(Losses),
	((Losses > 0,
	write($ But remember - because of you, $), write(Losses),
	write($ ducks lost their lives.$), nl,
	write($ One day, the ducklings will have revenge on you! $), nl);
	true).


/*
 *
 *
 *  FILENAME: x:\prosamp\ducks\ducky.pro
 *
 *  PARAMETERS:
 *
 *  DESCRIPTION:
 *
 *  RETURNS:
 *
 */
